@@ -127,6 +127,7 @@ void SddmKcm::save()
         } else {
             args[QStringLiteral("theme.conf.user/General/type")] = QStringLiteral("color");
         }
+        args[QStringLiteral("theme.conf.user/General/showClock")] = m_themesModel->data(currentThemeIndex, ThemesModel::ShowClockRole).toBool();
     }
     args[QStringLiteral("kde_settings.conf/Theme/Current")] = currentThemeIndex.data(ThemesModel::IdRole);
     args[QStringLiteral("kde_settings.conf/Autologin/User")] = m_data->sddmSettings()->user();
@@ -286,8 +287,9 @@ void SddmKcm::synchronizeSettings()
             }
         } else {
             qDebug() << "Synchronization successful";
-            Q_EMIT syncSuccessful();
         }
+
+        Q_EMIT syncAttempted();
     });
     job->start();
 }
@@ -320,6 +322,8 @@ void SddmKcm::resetSyncronizedSettings()
 
     args[QStringLiteral("kde_settings.conf/Theme/Font")] = QVariant();
 
+    args[QStringLiteral("theme.conf.user/General/showClock")] = true;
+
     KAuth::Action resetAction(QStringLiteral("org.kde.kcontrol.kcmsddm.reset"));
     resetAction.setHelperId(QStringLiteral("org.kde.kcontrol.kcmsddm"));
     resetAction.setArguments(args);
@@ -336,8 +340,9 @@ void SddmKcm::resetSyncronizedSettings()
             }
         } else {
             qDebug() << "Reset successful";
-            Q_EMIT resetSyncedDataSuccessful();
         }
+
+        Q_EMIT syncAttempted();
     });
     job->start();
 }
